with BasicTypes;

--# inherit BasicTypes;

package Time is
   ------------------------------------------------------------------
   -- Time
   --
   -- Description:
   --      Internal representation of time.
   --
   -----------------------------------------------------------------
   type T is private;

   ZeroTime : constant T;

   type YearsT is range 1901 .. 2099;
   type MonthsT is range 1 .. 12;
   type DaysT is range 1 .. 31;
   type HoursT is range 0 .. 23;
   type MinutesT is range 0 .. 59;
   ------------------------------------------------------------------
   -- Duration
   --
   -- Description:
   --    Duration in 1/10 seconds.
   --    The range allows 24 hours to be represented.
   ------------------------------------------------------------------
   type DurationT is range 0 .. 864000;

   ------------------------------------------------------------------
   -- Text
   -- DurationText
   --
   -- Description:
   --    Textual representation of time used in audit records.
   --    Times are displayed as "yyyy-mm-dd hh:mm:ss.s"
   --    Durations are displayed as "hh:mm:ss.s"
   ------------------------------------------------------------------
   subtype TextI is Positive range 1 .. 21;
   subtype TextT is String (TextI);

   subtype DurationTextI is Positive range 1 .. 10;
   subtype DurationTextT is String (DurationTextI);

   ------------------------------------------------------------------
   -- GreaterThan
   --
   -- Description:
   --    Boolean function implementing > on T.
   --
   -- Traceunit: C.Clock.GreaterThan
   --
   ------------------------------------------------------------------

   function GreaterThan (Left, Right : T) return Boolean;

   ------------------------------------------------------------------
   -- LessThan
   --
   -- Description:
   --    Boolean function implementing < on T.
   --
   -- Traceunit: C.Clock.LessThan
   --
   ------------------------------------------------------------------

   function LessThan (Left, Right : T) return Boolean;

   ------------------------------------------------------------------
   -- GreaterThanOrEqual
   --
   -- Description:
   --    Boolean function implementing >= on T.
   --
   -- Traceunit: C.Clock.GreaterThanOrEqual
   --
   ------------------------------------------------------------------

   function GreaterThanOrEqual (Left, Right : T) return Boolean;

   ------------------------------------------------------------------
   -- LessThanOrEqual
   --
   -- Description:
   --    Boolean function implementing <= on T.
   --
   -- Traceunit: C.Clock.LessThanOrEqual
   --
   ------------------------------------------------------------------

   function LessThanOrEqual (Left, Right : T) return Boolean;

   ------------------------------------------------------------------
   -- ConstructTime
   --
   -- Description:
   --    Constructs a time from raw constituent parts. If Time is not
   --    valid then the Success flag is set to false.
   --
   -- Traceunit: C.Clock.ConstructTime
   --


   ------------------------------------------------------------------
   -- SplitTime
   --
   -- Description:
   --    Decomposes a time into its constituent parts.
   --
   -- Traceunit: C.Clock.SplitTime
   --
   ------------------------------------------------------------------
   procedure SplitTime
     (TheTime : in T;
      Year    : out YearsT;
      Month   : out MonthsT;
      Day     : out DaysT;
      Hour    : out HoursT;
      Min     : out MinutesT);
   --# derives Year,
   --#         Month,
   --#         Day,
   --#         Hour,
   --#         Min   from TheTime;

   ------------------------------------------------------------------
   -- StartOfDay
   --
   -- Description:
   --    Converts Time to the start of the day.
   --
   -- Traceunit: C.Clock.StartOfDay
   --
   ------------------------------------------------------------------
   function StartOfDay (TheTime : T) return T;

   ------------------------------------------------------------------
   -- PrintDuration
   --
   -- Description:
   --    Converts a duration to Text for display.
   --
   -- Traceunit: C.Clock.PrintDuration
   --
   ------------------------------------------------------------------
   function PrintDuration (TheDuration : DurationT) return DurationTextT;

   ------------------------------------------------------------------
   -- PrintTime
   --
   -- Description:
   --    Converts a time to Text for display.
   --
   -- Traceunit: C.Clock.PrintText
   --
   ------------------------------------------------------------------
   function PrintTime (TheTime : T) return TextT;

   --     ------------------------------------------------------------------
   --     -- AddDuration
   --     --
   --     -- Description:
   --     --    Adds a duration to a time.
   --     --
   --     -- Traceunit: C.Clock.AddDuration
   --     --
   --     ------------------------------------------------------------------
   --     function AddDuration
   --       (TheTime : T; TheDuration : DurationT ) return T;

private

   MilliSecsInTenthSec : constant := 100;
   MilliSecsInSec      : constant := 1000;

   type MilliSecsT is range 0 .. DurationT'Last * MilliSecsInTenthSec - 1;

   type T is record
      Year     : YearsT;
      Month    : MonthsT;
      Day      : DaysT;
      MilliSec : MilliSecsT;
   end record;

   ZeroTime : constant T :=
      T'
     (Year     => YearsT'First,
      Month    => MonthsT'First,
      Day      => DaysT'First,
      MilliSec => MilliSecsT'First);

end Time;
